{% load i18n %}
{% load static %}
{% load bootstrap3 %}
<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8">
    <!-- Latest compiled and minified CSS -->
    <link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap.min.css">

    <!-- Optional theme -->
    <link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap-theme.min.css">
    <link rel="stylesheet" href="{% static 'css/example.css' %}">


    <script src="//ajax.googleapis.com/ajax/libs/jquery/2.1.0/jquery.min.js"></script>
    <!-- Latest compiled and minified JavaScript -->
    <script src="//netdna.bootstrapcdn.com/bootstrap/3.1.1/js/bootstrap.min.js"></script>

</head>
<body role="document">

<!-- Fixed navbar -->
<div class="navbar navbar-inverse navbar-fixed-top" role="navigation">
    <div class="container">
        <div class="navbar-header">
            <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
                <span class="sr-only">Toggle navigation</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a class="navbar-brand" href="/">django-plans example</a>
        </div>
        <div class="collapse navbar-collapse">
            <ul class="nav navbar-nav">
                <li><a href="{% url 'pricing' %}">Pricing</a></li>
                <li class="divider"></li>
                <li><a href="{% url 'foo_list' %}">List Foo</a></li>
                <li><a href="{% url 'foo_add' %}">Add Foo</a></li>
                <li class="divider"></li>
            </ul>
            <ul class="nav navbar-nav navbar-right">
                <li class="dropdown">
                    <a href="#" class="dropdown-toggle" data-toggle="dropdown">Profile <b class="caret"></b></a>
                    <ul class="dropdown-menu">
                        <li><a href="{% url 'current_plan' %}">Your account</a></li>
                        <li><a href="{% url 'upgrade_plan' %}">Upgrade plan</a></li>
                        <li class="divider"></li>
                        <li><a href="{% url 'order_list' %}">Orders</a></li>
                        <li><a href="{% url 'billing_info' %}">Billing data</a></li>
                        <li class="divider"></li>
                        <li><a href="{% url 'logout' %}">Logout</a></li>
                    </ul>
                </li>
            </ul>
        </div>
        <!--/.nav-collapse -->
    </div>
</div>




<div class="container">
    {% include "plans/expiration_messages.html" %}
    {% bootstrap_messages %}
    {% block body %}{% endblock %}
</div>
</body>
</html>
