{% extends "base.html" %}
{% load bootstrap3 %}
{% block body %}

    <div class="row">
        <div class="col-sm-6">

            <form action="" method="post" class="form">
                <legend>Sure to delete Foo "{{ object }}"?</legend>
                {% csrf_token %}
                {{ form.as_p }}
                {% buttons %}
                    <button type="submit" class="btn btn-danger">
                        {% bootstrap_icon "trash" %} Delete
                    </button>
                {% endbuttons %}
            </form>
        </div>
    </div>


{% endblock %}
