# Generated by Django 3.0.6 on 2020-06-09 14:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('plans', '0006_auto_20200504_1541'),
    ]

    operations = [
        migrations.AddField(
            model_name='recurringuserplan',
            name='card_masked_number',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
